-- ##### TerraLAB - www.terralab.ufop.br
-- Computer Science Departament, Federal Univeristy of Ouro Preto
-- TerraME Prey-Predator dynamic model based on differental equations
-- Tiago Garcia de Senna Carneiro - 07/2012
-------------
-- Each cell has different initial condition, i.e., initial prey and predator population size.
-- Different initial conditions result in differente dynamics.
-------------


--#PARAMETERS
birthPreyRate = 0.2
predationRate = 0.01 -- death prey rate
birthPredatorPerPreyRate = 0.01 -- birth predator rate
deathPredatorRate =  0.1 -- death prey rate
timeStep =  1 -- See the effects of 0.25, 0.125, 0.0625, and 0.03125
INTEGRATION_METHOD = integrationHeun --  vary the integration Method: ("integrationHeun"), "integrationRungeKutta", "integrationEuler"


--#SPATIAL MODEL
cs = CellularSpace{ xdim = 10 }
forEachCell( cs, function(cell) cell.preys = 100; cell.predators = 10; end )
cs:synchronize()

--#LEGENDS

preysLeg = Legend{
		-- Attribute name:  height
		type = "number",
		grouping = "equalsteps",
		slices = 100,
		precision = 10,
		stdDeviation = "none",
		maximum = 500,
		minimum = 0,
		colorBar = {
			{color = "white", value = 0},
			{color = "green", value = 500}
		}
}


predatorsLeg = Legend{
		-- Attribute name:  soilWater
		type = "number",
		grouping = "equalsteps",
		slices = 100,
		precision = 10,
		stdDeviation = "none",
		maximum = 500,
		minimum = 0,
		colorBar = {
			{color = "white", value = 0},
			{color = "red", value = 500}
		}
}

--#OBSERVERS
Observer{ subject = cs, type = "map", attributes = { "preys", "predators" }, legends = { preysLeg, predatorsLeg} }


--#RULES
for t = 0, 400, timeStep do 

	forEachCell( cs, function( cell )
		
		cell.preys, cell.predators = d{
			{ 	
				function( t, q )  
					return q[1]*birthPreyRate - q[1]*q[2]*predationRate 
				end , 
				function( t, q ) 
					return q[2]*q[1]*birthPredatorPerPreyRate - q[2]*deathPredatorRate
				end
			},
			{ cell.past.preys, cell.past.predators }, 
			0, timeStep, 0.03125 
		}
		
		cell:synchronize()
	end)
	
	cs:notify()
	
	print("Time: ", t)
end


